package com.example;

import akka.actor.typed.ActorRef;
import akka.actor.typed.Behavior;
import akka.actor.typed.javadsl.AbstractBehavior;
import akka.actor.typed.javadsl.ActorContext;
import akka.actor.typed.javadsl.Behaviors;
import akka.actor.typed.javadsl.Receive;

import java.util.ArrayList;
import java.util.List;

public class DeliveryCar extends AbstractBehavior<DeliveryCar.Message> {

    private final ArrayList<Paket> pakete = new ArrayList<>();
    private final ArrayList<ActorRef<Customer.Message>> route;
    private final int maxCapacity = 3;

    public interface Message {}

    public record LoadMessage(List<Paket> pakete) implements Message {}

    public record PickupResponse(Paket paket) implements Message {}

    public record ContinueRoute() implements Message {}

    private DeliveryCar(ActorContext<Message> context, ArrayList<ActorRef<Customer.Message>> route) {
        super(context);
        this.route = route;
    }

    public static Behavior<Message> create(List<ActorRef<Customer.Message>> route) {
        return Behaviors.setup(context -> new DeliveryCar(context, new ArrayList<>(route)));
    }

    @Override
    public Receive<Message> createReceive() {
        return newReceiveBuilder()
                .onMessage(LoadMessage.class, this::onLoadMessage)
                .onMessage(PickupResponse.class, this::onPickupResponse)
                .onMessage(ContinueRoute.class, msg -> {
                    deliverPackages();
                    return Behaviors.same();
                })
                .build();
    }

    private Behavior<Message> onLoadMessage(LoadMessage msg) {
        pakete.addAll(msg.pakete);
        checkNextStop();
        return this;
    }

    private Behavior<Message> onPickupResponse(PickupResponse rsp) {
        pakete.add(rsp.paket);
        getContext().getLog().info("Aktuelle Anzahl der Pakete im Truck: {}", pakete.size());
        checkNextStop();
        return this;
    }

    private void checkNextStop() {
        if (pakete.size() >= maxCapacity || route.isEmpty()) {
            getContext().getSelf().tell(new ContinueRoute());
        } else {
            sendPickupMessage();
        }
    }

    private void sendPickupMessage() {
        ActorRef<Customer.Message> nextCustomer = route.remove(0);
        nextCustomer.tell(new Customer.PickUpMessage(getContext().getSelf(), "Some string"));
    }

    private void deliverPackages() {
        ActorRef<Customer.Message> nextCustomer = route.get(0);
        for (Paket paket : pakete) {
            nextCustomer.tell(new Customer.DeliveryMessage(paket));
        }
        pakete.clear();
    }
}